/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was xdp_dbus_screencast_interface.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef XDP_DBUS_SCREENCAST_INTERFACE_H
#define XDP_DBUS_SCREENCAST_INTERFACE_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.freedesktop.portal.ScreenCast
 */
class OrgFreedesktopPortalScreenCastInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.freedesktop.portal.ScreenCast"; }

public:
    OrgFreedesktopPortalScreenCastInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgFreedesktopPortalScreenCastInterface();

    Q_PROPERTY(uint AvailableSourceTypes READ availableSourceTypes)
    inline uint availableSourceTypes() const
    { return qvariant_cast< uint >(property("AvailableSourceTypes")); }

    Q_PROPERTY(uint version READ version)
    inline uint version() const
    { return qvariant_cast< uint >(property("version")); }

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<QDBusObjectPath> CreateSession(const QVariantMap &options)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(options);
        return asyncCallWithArgumentList(QStringLiteral("CreateSession"), argumentList);
    }

    inline QDBusPendingReply<QDBusUnixFileDescriptor> OpenPipeWireRemote(const QDBusObjectPath &session_handle, const QVariantMap &options)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(session_handle) << QVariant::fromValue(options);
        return asyncCallWithArgumentList(QStringLiteral("OpenPipeWireRemote"), argumentList);
    }

    inline QDBusPendingReply<QDBusObjectPath> SelectSources(const QDBusObjectPath &session_handle, const QVariantMap &options)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(session_handle) << QVariant::fromValue(options);
        return asyncCallWithArgumentList(QStringLiteral("SelectSources"), argumentList);
    }

    inline QDBusPendingReply<QDBusObjectPath> Start(const QDBusObjectPath &session_handle, const QString &parent_window, const QVariantMap &options)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(session_handle) << QVariant::fromValue(parent_window) << QVariant::fromValue(options);
        return asyncCallWithArgumentList(QStringLiteral("Start"), argumentList);
    }

Q_SIGNALS: // SIGNALS
};

namespace org {
  namespace freedesktop {
    namespace portal {
      using ScreenCast = ::OrgFreedesktopPortalScreenCastInterface;
    }
  }
}

#endif
