// This file was generated by qtwaylandscanner
// source file is fake-input.xml

#ifndef QT_WAYLAND_FAKE_INPUT
#define QT_WAYLAND_FAKE_INPUT

#include "wayland-fake-input-client-protocol.h"
#include <QByteArray>
#include <QString>

struct wl_registry;

QT_BEGIN_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_GCC("-Wmissing-field-initializers")
QT_WARNING_DISABLE_CLANG("-Wmissing-field-initializers")

namespace QtWayland {
    class  org_kde_kwin_fake_input
    {
    public:
        org_kde_kwin_fake_input(struct ::wl_registry *registry, uint32_t id, int version);
        org_kde_kwin_fake_input(struct ::org_kde_kwin_fake_input *object);
        org_kde_kwin_fake_input();

        virtual ~org_kde_kwin_fake_input();

        void init(struct ::wl_registry *registry, uint32_t id, int version);
        void init(struct ::org_kde_kwin_fake_input *object);

        struct ::org_kde_kwin_fake_input *object() { return m_org_kde_kwin_fake_input; }
        const struct ::org_kde_kwin_fake_input *object() const { return m_org_kde_kwin_fake_input; }
        static org_kde_kwin_fake_input *fromObject(struct ::org_kde_kwin_fake_input *object);

        bool isInitialized() const;

        uint32_t version() const;
        static const struct ::wl_interface *interface();

        void authenticate(const QString &application, const QString &reason);
        void pointer_motion(wl_fixed_t delta_x, wl_fixed_t delta_y);
        void button(uint32_t button, uint32_t state);
        void axis(uint32_t axis, wl_fixed_t value);
        void touch_down(uint32_t id, wl_fixed_t x, wl_fixed_t y);
        void touch_motion(uint32_t id, wl_fixed_t x, wl_fixed_t y);
        void touch_up(uint32_t id);
        void touch_cancel();
        void touch_frame();
        void pointer_motion_absolute(wl_fixed_t x, wl_fixed_t y);
        void keyboard_key(uint32_t button, uint32_t state);
        void destroy();
        void keyboard_keysym(uint32_t keysym, uint32_t state);

    private:
        struct ::org_kde_kwin_fake_input *m_org_kde_kwin_fake_input;
    };
}

QT_WARNING_POP
QT_END_NAMESPACE

#endif
