/* Generated by wayland-scanner 1.25.0 */

/*
 * SPDX-FileCopyrightText: 2020-2021 Aleix Pol Gonzalez <aleixpol@kde.org>
 *
 * SPDX-License-Identifier: LGPL-2.1-or-later
 */

#include <stdbool.h>
#include <stdlib.h>
#include <stdint.h>
#include "wayland-util.h"

#ifndef __has_attribute
# define __has_attribute(x) 0  /* Compatibility with non-clang compilers. */
#endif

#if (__has_attribute(visibility) || defined(__GNUC__) && __GNUC__ >= 4)
#define WL_PRIVATE __attribute__ ((visibility("hidden")))
#else
#define WL_PRIVATE
#endif

extern const struct wl_interface wl_output_interface;
extern const struct wl_interface zkde_screencast_stream_unstable_v1_interface;

static const struct wl_interface *zkde_screencast_unstable_v1_types[] = {
	NULL,
	&zkde_screencast_stream_unstable_v1_interface,
	&wl_output_interface,
	NULL,
	&zkde_screencast_stream_unstable_v1_interface,
	NULL,
	NULL,
	&zkde_screencast_stream_unstable_v1_interface,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	&zkde_screencast_stream_unstable_v1_interface,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	&zkde_screencast_stream_unstable_v1_interface,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
};

static const struct wl_message zkde_screencast_unstable_v1_requests[] = {
	{ "stream_output", "nou", zkde_screencast_unstable_v1_types + 1 },
	{ "stream_window", "nsu", zkde_screencast_unstable_v1_types + 4 },
	{ "destroy", "", zkde_screencast_unstable_v1_types + 0 },
	{ "stream_virtual_output", "2nsiifu", zkde_screencast_unstable_v1_types + 7 },
	{ "stream_region", "3niiuufu", zkde_screencast_unstable_v1_types + 13 },
	{ "stream_virtual_output_with_description", "4nssiifu", zkde_screencast_unstable_v1_types + 20 },
};

WL_PRIVATE const struct wl_interface zkde_screencast_unstable_v1_interface = {
	"zkde_screencast_unstable_v1", 5,
	6, zkde_screencast_unstable_v1_requests,
	0, NULL,
};

static const struct wl_message zkde_screencast_stream_unstable_v1_requests[] = {
	{ "close", "", zkde_screencast_unstable_v1_types + 0 },
};

static const struct wl_message zkde_screencast_stream_unstable_v1_events[] = {
	{ "closed", "", zkde_screencast_unstable_v1_types + 0 },
	{ "created", "u", zkde_screencast_unstable_v1_types + 0 },
	{ "failed", "s", zkde_screencast_unstable_v1_types + 0 },
};

WL_PRIVATE const struct wl_interface zkde_screencast_stream_unstable_v1_interface = {
	"zkde_screencast_stream_unstable_v1", 5,
	1, zkde_screencast_stream_unstable_v1_requests,
	3, zkde_screencast_stream_unstable_v1_events,
};

